function [w, T, bias] = SCutFBR1(train_data, options, train_final)
    % SCutFBR.1 in the paper
    % It uses two-level cross-validation to select fbr and threshold.

    % check input
    if (nargin ~= 3)
        if (nargin < 3)
            train_final = true;
        end
        if (nargin < 2)
            options = '-q -B 1';
        end
        if (nargin < 1 || nargin > 3)
            fprintf('Incorrect number of input parameters\n');
            return;
        end
    end
    
    x = train_data.x;
    y = train_data.y;
    map = train_data.map;

    % bias option
    bias = get_bias(options);
    if bias > 0
        w_size = size(x, 1) + 1;
    else
        w_size = size(x, 1);
    end
    
    t0all = clock;

    % number of label = number of model to be trained
    L = size(y, 2); 
    
    w = zeros(w_size, L);
    T = zeros(L, 1);

    % obtain (w,T) for each label
    % parpool(4);
    parfor i = 1:L
        fprintf(1, 'INFO: training %dth label (%d)...\n', i, map(i));
        t0 = clock;
        [w(:, i), T(i)] = train_one_label(2*y(:, i)-1, x, options, train_final);
        fprintf(1, '\tINFO: Done in %.6f seconds\n', etime(clock, t0));
    end

    fprintf(1, 'INFO: Total Time: %.6f seconds\n', etime(clock, t0all));
end

function [w, T] = train_one_label(proby, probx, options, train_final)
    % size(x) = (#label, #sample)

    fbr_list = [0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8];

    nr_fold = 3;

    bias = get_bias(options);

    warning('off', 'stats:cvpartition:KFoldMissingGrp');
    cv = cvpartition(proby, 'KFold', nr_fold, 'Stratify', true);

    f_list = zeros(size(fbr_list));

    % cv
    for fold = 1:nr_fold
        train_id = cv.training(fold);
        valid_id = cv.test(fold);

        train_y = proby(train_id);
        train_x = probx(:, train_id);
        valid_y = proby(valid_id);
        valid_x = probx(:, valid_id);

        % scutfbr
        [scutfbr_w, scutfbr_b_list] = scutfbr(train_y, train_x, fbr_list, options, true);
        if bias > 0
            valid_x = [valid_x; ones(1, size(valid_x, 2)) * bias];
        end
        wTx = valid_x' * scutfbr_w;
        
        F = fmeasure(valid_y, 2*(wTx > scutfbr_b_list')-1);
        f_list = f_list + F;
    end

    best_fbr = fbr_list(find(f_list == max(f_list), 1, 'last'));
    if max(f_list) == 0
        best_fbr = min(fbr_list);
        % fprintf(1, '\tINFO: train_one_label: F all 0\n');
    end

    % final model
    [w, T] = scutfbr(proby, probx, best_fbr, options, train_final);
end



