function [w, T_list] = scutfbr(proby, probx, fbr_list, options, train_final)
    % Given a list of fbr values, produce the thresholds
    % corresponding to the the fbr values using CV.
    
    T_list = zeros(size(fbr_list, 2), 1);

    nr_fold = 3;

    bias = get_bias(options);
    
    warning('off', 'stats:cvpartition:KFoldMissingGrp');
    cv = cvpartition(proby, 'KFold', nr_fold, 'Stratify', true);

    % cv
    for fold = 1:nr_fold
        train_id = cv.training(fold);
        valid_id = cv.test(fold);

        train_y = proby(train_id);
        train_x = probx(:, train_id);
        valid_y = proby(valid_id);
        valid_x = probx(:, valid_id);

        % scut
        scut_w = do_train(train_y, train_x, options);
        if (bias > 0)
            valid_x = [valid_x; ones(1, size(valid_x, 2)) * bias];
        end
        wTx = valid_x' *  scut_w; % shape = (#sample in valid)
        scut_b = 0;

        start_F = fmeasure(valid_y, 2*(wTx > scut_b)-1);

        [sorted_wTx, sorted_wTx_index] = sort(wTx, 1, 'ascend'); 

        % best_F is initialized as predicting all as 1,
        % that is, the threshhold is set to the lowest
        tp = sum(valid_y == 1); fp = size(valid_y,1)-tp; fn = 0;
        cut = -1;
        best_F = (2*tp) / (2*tp + fp + fn);
        for i = 1:size(valid_y, 1)
            if valid_y(sorted_wTx_index(i)) == -1,
                fp = fp - 1;
            else
                tp = tp - 1; fn = fn + 1;
            end
            F = (2*tp) / (2*tp + fp + fn);
            
            if F >= best_F
                best_F = F;
                cut = i;
            end
        end

        % modify b
        if best_F > start_F
            if cut == -1 % i.e., all +1
                val = sorted_wTx(1);
                scut_b = (val - eps(val)); % predict all +1
            elseif cut == size(valid_y, 1)
                val = sorted_wTx(size(valid_y, 1));
                scut_b = (val + eps(val));
            else
                scut_b = (sorted_wTx(cut) + sorted_wTx(cut+1))/2;
            end
        end

        F = fmeasure(valid_y, 2*(wTx > scut_b)-1);

        % compare to fbr_list
        for i = 1:size(fbr_list, 2)
            if F > fbr_list(i)
                T_list(i) = T_list(i) + scut_b;
            else
                T_list(i) = T_list(i) + max(wTx);
            end
        end
    end

    % final model
    T_list = T_list / nr_fold;
    if train_final
        w = do_train(proby, probx, options);
    else
        if bias > 0
            w_size = size(probx, 1) + 1;
        else
            w_size = size(probx, 1);
        end
        w = zeros(w_size, 1);
    end
end

