% This experiment count the occurences of situations like Figure 1 and Figure 2
% by performing 3-fold CV on labels in different frequency ranges.
% the result may very significantly depending on the random seed
addpath('utils', 'algos')
run('data_paths.m')

% see data_paths.m for the dataname to use
dataname = 'blogcat';
paths = datasets(dataname);

train_path = paths{1};
options = '-q -B 1';
options = [options ' -s 2'];
nr_fold = 3;
fbr = 0.4;

[y, x, map] = libsvmread_ml(train_path);

probx = x';

bias = get_bias(options);

counts = sum(y,1);

%% get rare labels and counts Figure 1
rare = find(counts < 10);
fig1_counts = zeros(numel(rare), 1);

parfor j = 1:numel(rare)
    fprintf("%d/%d\n", j, numel(rare));
    i = rare(j);
    proby = y(:, i) * 2 - 1;

    warning('off', 'stats:cvpartition:KFoldMissingGrp');
    cv = cvpartition(proby, 'KFold', nr_fold, 'Stratify', true);
    for fold = 1:nr_fold
        train_id = cv.training(fold);
        valid_id = cv.test(fold);

        train_y = proby(train_id);
        train_x = probx(:, train_id);
        valid_y = proby(valid_id);
        valid_x = probx(:, valid_id);

        % scut
        scut_w = do_train(train_y, train_x, options);
        if bias > 0
            valid_x = [valid_x; ones(1, size(valid_x, 2)) * bias];
        end
        wTx = valid_x' *  scut_w; % shape = (#sample in valid)

        [sorted_wTx, sorted_wTx_index] = sort(wTx, 1, 'ascend'); 
        
        % count figure 2 situation
        sorted_y = valid_y(sorted_wTx_index);
        highest_positive = find(sorted_y > 0, 1, 'last');
        pos_max = sorted_wTx(highest_positive);

        % count false positives for the threshold immediately below pos_max
        fp = sum(sorted_y(highest_positive:end) < 0);
        fprintf("pos_max: %f, fp: %f\n", pos_max, fp);
        
        if (pos_max < -0.5) & fp > 100 % low threshold that gives a lot of false positive
            fig1_counts(j) = fig1_counts(j) + 1;
        end
    end
end

%% get medium to frequent labels and counts Figure 2
medium = find(counts >= 10 & counts <= 1000);
% medium = find(counts);
fig2_counts = zeros(numel(medium), 1);

parfor j = 1:numel(medium)
    fprintf("%d/%d\n", j, numel(medium));
    i = medium(j);
    proby = y(:, i) * 2 - 1;
    
    warning('off', 'stats:cvpartition:KFoldMissingGrp');
    cv = cvpartition(proby, 'KFold', nr_fold, 'Stratify', true);
    for fold = 1:nr_fold
        train_id = cv.training(fold);
        valid_id = cv.test(fold);

        train_y = proby(train_id);
        train_x = probx(:, train_id);
        valid_y = proby(valid_id);
        valid_x = probx(:, valid_id);

        % scut
        scut_w = do_train(train_y, train_x, options);
        if bias > 0
            valid_x = [valid_x; ones(1, size(valid_x, 2)) * bias];
        end
        wTx = valid_x' *  scut_w; % shape = (#sample in valid)

        [sorted_wTx, sorted_wTx_index] = sort(wTx, 1, 'ascend'); 
        sorted_y = valid_y(sorted_wTx_index);

        tp = sum(valid_y == 1); fp = size(valid_y,1)-tp; fn = 0;
        cut = -1;
        highest_positive = -1;
        best_F = (2*tp) / (2*tp + fp + fn);
        for k = 1:size(sorted_y, 1)
            if sorted_y(k) == -1
                fp = fp - 1;
            else
                highest_positive = k;
                tp = tp - 1; fn = fn + 1;
            end
            F = (2*tp) / (2*tp + fp + fn);
            
            if F >= best_F
                best_F = F;
                cut = k;
            end
        end
        
        if highest_positive == -1
            continue;
        end
        % count false positive if threshold is lower than pos_max
        % if it is 0, then we have easy positives
        fp = sum(sorted_y(highest_positive:end) < 0);
        fprintf("best_F: %f, fp: %f\n", best_F, fp);
        
        if fp == 0 & best_F < fbr
            fig2_counts(j) = fig2_counts(j) + 1;
        end
    end
end

fprintf(1, "Fig1 occured for %.2f%% of validation sets.\n", sum(fig1_counts) / numel(fig1_counts) / nr_fold * 100);
fprintf(1, "Fig1 affected %.2f%% of labels.\n", sum(fig1_counts > 0) / numel(fig1_counts) * 100);
fprintf(1, "Fig2 occured for %.2f%% of validation sets.\n", sum(fig2_counts) / numel(fig2_counts) / nr_fold * 100);
fprintf(1, "Fig2 affected %.2f%% of labels.\n", sum(fig2_counts > 0) / numel(fig2_counts) * 100);
